library(caTools) #numerical integration (using trapezoidal rule)
library(xtable)  #write tables in latex format

source("./Functions/bound_functions.R");source("./Functions/plotAll.R")

JCData <- read.csv("./Data/CleanedJCData.csv",header = TRUE, as.is = TRUE)

JCData$earnq16 <- as.numeric(JCData$earnq16)
JCData <- JCData[!is.na(JCData$earnq16)&!is.na(JCData$z),]
JCData$emp <- JCData$earnq16 > 0
n <- nrow(JCData)

cohort <- c("all","female","male")

## flag for dependent variable
## TRUE for employment status, FALSE for weekly wage
emp.var <- !TRUE

## truncation level (among the treated compliers)
q <- 1

for(j in cohort){

  cmp <- JCData$z > 0 & JCData$treatmnt #treated compliers
  sex <- ifelse(rep(j,n)=="female",!!JCData$female,ifelse(rep(j,n)=="male",!JCData$female,rep(TRUE,n)))
  sample.ind <- cmp & sex

  if(emp.var){
    JCData$y <- JCData$emp
    b <- 0.0017 ## (0.024/30)*2
    ey0 <- weighted.mean(JCData$y[(!JCData$treatmnt) & (JCData$z==0) & sex],JCData$wgt48b[(!JCData$treatmnt) & (JCData$z==0) & sex])
    plotAll(JCData[sample.ind&(JCData$z<=quantile(JCData$z, q)),],j, b, ey0, emp.var)
  }
      
  if(!emp.var){
    JCData$y <- JCData$earnq16
    b <- ifelse(j=="female",0.92,ifelse(j=="male", 1.3, 1.14))
    ## all: (200.2 * 0.12 / 30) + (200.2 * 0.00165)  ~  1.13113 -> 1.4
    ## male: (227.5 * 0.12 / 30) + (227.5 * 0.00165) ~ 1.285375 -> 1.3
    ## female: (161.1 * 0.12 / 30) + (161.1 * 0.00165) ~ 0.910215 -> 0.92
    ey0 <- weighted.mean(JCData$y[(!JCData$treatmnt) & (JCData$z==0) & sex],JCData$wgt48b[(!JCData$treatmnt) & (JCData$z==0) & sex])
    plotAll(JCData[sample.ind&(JCData$z<quantile(JCData$z, q)),],j, b, ey0, emp.var)
  }
} 