# Data

t = seq(from=8, to=20, by=1) # Treatment
Eyz = c(2.249,2.302,2.195,2.346,2.496,2.658,2.639,2.693,2.870,2.775,3.006,3.009,2.936) # Values for E[Y|Z=z]
pz = c(0.014,0.018,0.018,0.025,0.413,0.074,0.083,0.035,0.189,0.038,0.051,0.020,0.021)  # Values for P(Z=z)
Ey = sum(Eyz*pz)

b=0.2 # Smoothness parameter

ymin = 1.4 # Value of y_min
ymax = 5.0 # Value of y_max


# Function calculating STR bounds

str = function(ey,p,t,b){

ubd = rep(0,length(t))
lbd = rep(0,length(t))

for(i in 1:length(t)){
temp = abs(t[i]-t)
temp = sum(temp*p)

ubd[i] = ey + b*temp
lbd[i] = ey - b*temp

}

res = matrix(c(lbd,ubd),nrow=length(t),ncol=2)
res

}


# Function calculating SMTR bounds

smtr = function(ey,p,t,b){

ubd = rep(0,length(t))
lbd = rep(0,length(t))

for(i in 1:length(t)){
temp = t-t[i]
tempu = temp*(temp<0)*(-1)
templ = temp*(temp>0)
tempu = sum(tempu*p)
templ = sum(templ*p)

ubd[i] = ey + b*tempu
lbd[i] = ey - b*templ

}

res = matrix(c(lbd,ubd),nrow=length(t),ncol=2)
res

}


# Function calculating MTR bounds

mtr = function(eyz,p,t,ymin,ymax){

ubd = rep(0,length(t))
lbd = rep(0,length(t))

for(i in 1:length(t)){


temp4 = ymax

temp8 = ymin

ubd[i] = sum(temp4*p*(t<t[i])+eyz*p*(t>=t[i]))
lbd[i] = sum(temp8*p*(t>t[i])+eyz*p*(t<=t[i]))

}

res = matrix(c(lbd,ubd),nrow=length(t),ncol=2)
res

}


# Function outputs for lower and upper bounds for each type of assumptions

l_str = str(Ey,pz,t,b)[,1]
u_str = str(Ey,pz,t,b)[,2]
l_smtr = smtr(Ey,pz,t,b)[,1]
u_smtr = smtr(Ey,pz,t,b)[,2]
l_mtr = mtr(Ey,pz,t,ymin,ymax)[,1]
u_mtr = mtr(Ey,pz,t,ymin,ymax)[,2]


# Plot

pdf("MTRcomp.pdf")

plot(t, u_smtr, type="l",ylab="Log Wage", lty=5, lwd=1, xlab="Years of Education",ylim=c(min(l_mtr),max(u_mtr)))
lines(t, u_mtr, type="l",lty=1)
lines(t, l_mtr, type="l",lty=1)
lines(t, l_smtr, type="l",lty=5,lwd=1)
lines(t, l_str, type="l",lty=2,lwd=1)
lines(t, u_str, type="l",lty=2,lwd=1)
legend(8,5,c("MTR","STR","SMTR"),lty=c(1,2,5))

dev.off()
