# Function for calculating bounds for treatment effect under SMTR-MTS

SMTRMTS_TE = function(b,tvec,t1,t2,eyz,fz){
  
  ind1 = which(tvec==t1)
  ind2 = which(tvec==t2)
  
  fi = function(s){
    
    inds = which(tvec==s)
    res = eyz[inds]+b*(t2-s)
    
    for(i in inds:ind2){
      newres = eyz[i]+b*(t2-tvec[i])
      res = min(res,newres)
    }
    
    res
    
  }
  
  fs = function(s){
    
    inds = which(tvec==s)
    res = eyz[ind1]
    
    for(i in ind1:inds){
      newres = eyz[i]-b*(tvec[i]-t1)
      res = max(res,newres)
    }
    
    res
  }
  
  if(ind1==1){
    
    comp1 = 0
    
  }else{
    
    comp1 = numeric(length(tvec[1:(ind1-1)]))
    
    for(i in 1:(ind1-1)){
      
      comp1[i] = min(fi(tvec[i])-eyz[i],b*(t2-t1))
      
    }
    
  }
  
  
  comp2 = numeric(length(tvec[ind1:ind2]))
  
  for(i in ind1:ind2){
    
    comp2[i-ind1+1] = min(fi(tvec[i])-fs(tvec[i]),b*(t2-t1))
    
  }
  
  if(ind2==length(tvec)){
    comp3 = 0
  }else{
    
    comp3 = numeric(length(tvec[(ind2+1):length(tvec)]))
    
    for(i in (ind2+1):length(tvec)){
      
      comp3[i-ind2] = min(eyz[i]-fs(tvec[i]),b*(t2-t1))
      
    }
    
    
    
  }
  
  
  ub = ifelse(ind1==1,0,
              sum(comp1*fz[1:(ind1-1)]))+sum(comp2*fz[ind1:ind2])+ifelse(ind2==length(tvec),0,
                                                                         sum(comp3*fz[(ind2+1):length(tvec)]))
  
  res = ub
  
  res
  
}


#Function for calculating MTR-MTS bounds

MTRMTS = function(tvec,t,eyz,fz){ 
  
  ind = which(tvec==t)
  if(ind==1){
    ub = sum(eyz[ind:length(tvec)]*fz[ind:length(tvec)])
    lb = eyz[ind]
    
  }else if(ind==length(tvec)){
    ub = eyz[ind]
    lb = sum(eyz[1:ind]*fz[1:ind])
    
  }else{
    ub = eyz[ind]*sum(fz[1:ind]) + sum(eyz[(ind+1):length(tvec)]*fz[(ind+1):length(tvec)])
    lb = sum(eyz[1:(ind-1)]*fz[1:(ind-1)]) + eyz[ind]*sum(fz[ind:length(tvec)]) 
    
  }
  
  res = c(lb,ub)
  res
}


# Data

t = seq(from=8, to=20, by=1) # Treatment
eyz = c(2.249,2.302,2.195,2.346,2.496,2.658,2.639,2.693,2.870,2.775,3.006,3.009,2.936) # Values for E[Y|Z=z]
pz = c(0.014,0.018,0.018,0.025,0.413,0.074,0.083,0.035,0.189,0.038,0.051,0.020,0.021)  # Values for P(Z=z)
Ey = sum(Eyz*pz)

eyz_s=sort(eyz) # We sort the estimates of g(t,t) - see section 5.2.

bgrid = seq(from=0,to=0.4,by=0.01)
resvec = numeric(length(bgrid))


# Figure for TE(12,16)
for(i in 1:length(bgrid)){
  
  resvec[i] = SMTRMTS_TE(bgrid[i],t,12,16,eyz_s,pz)
  
}

pdf("TE_12-16.pdf")
plot(bgrid,resvec,type="l",ylim=c(0,0.5),xlab="Choice of b",ylab="Treatment effect")
lines(bgrid,4*bgrid,lty=2)
abline(h=MTRMTS(t,16,eyz_s,pz)[2]-MTRMTS(t,12,eyz_s,pz)[1],lty=3)

legend("topright",lty=c(1,3,2),c("SMTR-MTS","MTR-MTS","4b"))

dev.off()


# Figure for TE(16,18)
for(i in 1:length(bgrid)){
  
  resvec[i] = SMTRMTS_TE(bgrid[i],t,16,18,eyz_s,pz)
  
}

pdf("TE_16-18.pdf")
plot(bgrid,resvec,type="l",ylim=c(0,0.5),xlab="Choice of b",ylab="Treatment effect")
lines(bgrid,2*bgrid,lty=2)
abline(h=MTRMTS(t,18,eyz_s,pz)[2]-MTRMTS(t,16,eyz_s,pz)[1],lty=3)

legend("topright",lty=c(1,3,2),c("SMTR-MTS","MTR-MTS","4b"))

dev.off()