# Data

t = seq(from=8, to=20, by=1) # Treatment
Eyz = c(2.249,2.302,2.195,2.346,2.496,2.658,2.639,2.693,2.870,2.775,3.006,3.009,2.936) # Values for E[Y|Z=z]
pz = c(0.014,0.018,0.018,0.025,0.413,0.074,0.083,0.035,0.189,0.038,0.051,0.020,0.021)  # Values for P(Z=z)
Ey = sum(Eyz*pz)

a=0.4 # Smoothness parameter

ymin = 1.4 # Value of y_min
ymax = 5.0 # Value of y_max


# Function for calculating STS bounds   
sts = function(eyz,p,t,a){

ubd = rep(0,length(t))
lbd = rep(0,length(t))

for(i in 1:length(t)){
temp = abs(t[i]-t)
temp = sum(temp*p)

ubd[i] = eyz[i] + a*temp
lbd[i] = eyz[i] - a*temp

}
res = matrix(c(lbd,ubd),nrow=length(t),ncol=2)
res

}


# Function for calculating SMTS bounds
smts = function(eyz,p,t,a){

ubd = rep(0,length(t))
lbd = rep(0,length(t))

for(i in 1:length(t)){
temp = t-t[i]
templ = temp*(temp<0)*(-1)
tempu = temp*(temp>0)
templ = sum(templ*p)
tempu = sum(tempu*p)

ubd[i] = eyz[i] + a*tempu
lbd[i] = eyz[i] - a*templ

}

res = matrix(c(lbd,ubd),nrow=length(t),ncol=2)
res

}


# Function for calculating MTS bounds
mts = function(eyz,p,t,ymin,ymax){

ubd = rep(0,length(t))
lbd = rep(0,length(t))

for(i in 1:length(t)){


temp4 = ymax
temp8 = ymin

ubd[i] = sum(temp4*p*(t>t[i])+eyz[i]*p*(t<=t[i]))
lbd[i] = sum(temp8*p*(t<t[i])+eyz[i]*p*(t>=t[i]))

}

res = matrix(c(lbd,ubd),nrow=length(t),ncol=2)
res

}


# Function outputs for lower and upper bounds for each type of assumptions
l_mts = mts(Eyz,pz,t,ymin,ymax)[,1]
u_mts = mts(Eyz,pz,t,ymin,ymax)[,2]
l_sts = sts(Eyz,pz,t,a)[,1]
u_sts = sts(Eyz,pz,t,a)[,2]
l_smts = smts(Eyz,pz,t,a)[,1]
u_smts = smts(Eyz,pz,t,a)[,2]


# Plot results

pdf("MTScomp.pdf")

plot(t, u_smts, type="l",ylab="Log Wage", lty=5, xlab="Years of Education",ylim=c(min(l_mts,l_sts),max(u_mts,u_sts)))
lines(t, u_mts, type="l",lty=1)
lines(t, l_mts, type="l",lty=1)
lines(t, l_smts, type="l",lty=5)
lines(t, l_sts, type="l",lty=2)
lines(t, u_sts, type="l",lty=2)
legend(13,1,c("MTS","STS","SMTS"),lty=c(1,2,5))

dev.off()



