ssc install clrbound
ssc install moremata

use "Data\NLSY.dta", clear

set more off

/* Preserve only relevant variables */
keep afqt eduyr lnwage


/*Storage for SMTR+MIV */
gen lbd_20 = .
gen ubd_20 = .
gen lbd_15 = .
gen ubd_15 = .
gen lbd_10 = .
gen ubd_10 = .

/*Storage for MTR+MIV */
gen lbd2 =.
gen ubd2 =.

gen t = . /* Grid for different years of schooling */
forvalues i = 1/13{
replace t = `i' + 7 in `i'
}


/* Generating grid for afqt which will be used in clrbound2 command */
/* 100 numbers between the 5th and the 95th quantile of afqt will be used */

gen vigrid =. 

sum afqt, detail

scalar a1 = r(p5)
scalar b1 = r(p95)
scalar k1 = (b1-a1)/100
replace vigrid = a1 in 1
forvalues i = 2/101{
replace vigrid = a1+k1*(`i'-1) in `i'
}

gen vigrid2 = vigrid

/* Focus of analysis is when conditional on the median afqt score */

local thd = 0

replace vigrid =. if vigrid > `thd'
replace vigrid2 =. if vigrid2 < `thd'


/* The number used in MP NBER paper */
local ymin = 1.4
local ymax = 5.0

/* Different smoothness parameters to be tested */

local b1 = 0.2
local b2 = 0.15
local b3 = 0.1

/* Inference Part*/

forvalues i = 1/13{

local k = `i' + 7

spid lnwage eduyr afqt vigrid vigrid2, assum("smtr") b(`b1') t(`k') level(0.95) method("series") 

replace lbd_20 = e(lbd) in `i'
replace ubd_20 = e(ubd) in `i'

spid lnwage eduyr afqt vigrid vigrid2, assum(smtr) b(`b2') t(`k') level(0.95) method("series") 

replace lbd_15 = e(lbd) in `i'
replace ubd_15 = e(ubd) in `i'

spid lnwage eduyr afqt vigrid vigrid2, assum(smtr) b(`b3') t(`k') level(0.95) method("series") 

replace lbd_10 = e(lbd) in `i'
replace ubd_10 = e(ubd) in `i'


/* MTR + MIV dependent variables */
gen yl2 = lnwage*(1-(eduyr>t[`i']))+`ymin'*(eduyr>t[`i'])
gen yu2 = lnwage*(1-(eduyr<t[`i']))+`ymax'*(eduyr<t[`i'])

clr3bound ((yl2 afqt vigrid))((yu2 afqt vigrid2)), level(0.95) method("series")

replace lbd2 = e(lbd) in `i'
replace ubd2 = e(ubd) in `i'

drop yl2
drop yu2

}



/* Graphing */

twoway (rline lbd_20 ubd_20 t)(rline lbd2 ubd2 t)

keep lbd* ubd* t
save Result_SMTR-MIV, replace
export delimited using "Result_SMTR-MIV.raw", delimiter(tab) replace
* For STATA version lower than 13, use: outsheet using "Result_SMTR-MIV,raw", replace 
