#################################################################################
########### Functions for Obtaining Identification Region #######################
#################################################################################


MTR = function(tvec,t,eyz,fz,ymax,ymin){ # MTR bounds
  ind = which(tvec==t)
  if(ind==1){
	ub = trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
	lb = ymin*trapz(tvec[ind:length(tvec)],fz[ind:length(tvec)]) 
  }else if(ind==length(tvec)){
	ub = ymax*trapz(tvec[1:ind],fz[1:ind])
	lb = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind]) 
  }else{
	ub = ymax*trapz(tvec[1:ind],fz[1:ind]) + trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
	lb = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind]) + ymin*trapz(tvec[ind:length(tvec)],fz[ind:length(tvec)]) 
  }
  res = c(lb,ub)
  res
}


MTR1 = function(t,y,z,ymax,ymin,wgt){ # MTR bounds using sample analogue
	
	ub = ymax*weighted.mean(z<t,w=wgt) + weighted.mean(y[z>=t],wgt[z>=t])*weighted.mean(z>=t,w=wgt)
	lb = ymin*weighted.mean(z>=t,w=wgt) + weighted.mean(y[z<=t],wgt[z<=t])*weighted.mean(z<=t,w=wgt)
	res = c(lb,ub)
	res
}


SMTR = function(b,tvec,t,eyz,fz){ # SMTR bounds
  ind = which(tvec==t)
  if(ind==1){
  ub = trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
  lb = trapz(tvec[ind:length(tvec)],(eyz[ind:length(tvec)]+b*(t-tvec[ind:length(tvec)]))*fz[ind:length(tvec)]) 
  
  }else if(ind==length(tvec)){
  ub = trapz(tvec[1:ind],(eyz[1:ind]+b*(t-tvec[1:ind]))*fz[1:ind]) 
  lb = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind]) 
  
  }else{
  ub = trapz(tvec[1:ind],(eyz[1:ind]+b*(t-tvec[1:ind]))*fz[1:ind]) + trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
  lb = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind]) + trapz(tvec[ind:length(tvec)],(eyz[ind:length(tvec)]+b*(t-tvec[ind:length(tvec)]))*fz[ind:length(tvec)]) 
  
  }
    
  res = c(lb,ub)
  res
}


SMTR1 = function(b,t,y,z,ymax,ymin,wgt){ # SMTR bounds using sample analogue
	
	ucomp = (t-z)*(z<t)
	ub = weighted.mean(y,w=wgt)+b*weighted.mean(ucomp,w=wgt)
	lcomp = (z-t)*(z>t)
	lb = weighted.mean(y,w=wgt)-b*weighted.mean(lcomp,w=wgt)
	res = c(lb,ub)
	res
}

MTRMTS = function(tvec,t,eyz,fz){ #MTR-MTS bounds
 
 ind = which(tvec==t)
  if(ind==1){
  ub = trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
  lb = eyz[ind]*trapz(tvec[ind:length(tvec)],fz[ind:length(tvec)]) 
  
  }else if(ind==length(tvec)){
  ub = eyz[ind]*trapz(tvec[1:ind],fz[1:ind]) 
  lb = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind])
  
  }else{
  ub = eyz[ind]*trapz(tvec[1:ind],fz[1:ind]) + trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
  lb = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind]) + eyz[ind]*trapz(tvec[ind:length(tvec)],fz[ind:length(tvec)]) 
  
  }
  
  res = c(lb,ub)
  res
}



MTRMTS1 = function(tvec,t,y,z,eyz,wgt){ #MTR-MTS bounds using sample analogue as much as possible
  
	ind = which(tvec==t)
	
	
	ub = eyz[ind]*weighted.mean(z<=t,w=wgt) + weighted.mean(y[z>t],w=wgt)*weighted.mean(z>t,w=wgt)
	lb = eyz[ind]*weighted.mean(z>=t,w=wgt) + weighted.mean(y[z<t],w=wgt)*weighted.mean(z<t,w=wgt)

	res = c(lb,ub)
	res
  
}

SMTRMTS = function(b,tvec,t,eyz,fz){ #SMTR-MTS bounds
  
  ind = which(tvec==t)
  inf = numeric(ind)
  
  for(i in 1:ind){
    min = eyz[i]+b*(t-tvec[i])
    
    for(j in i:ind){
      newmin = eyz[j]+b*(t-tvec[j])
      if(newmin<min) min=newmin
    }
    
    inf[i] = min
  }  

  
  sup = numeric(length(tvec[ind:length(tvec)]))
  
  for(i in ind:length(tvec)){
    max = eyz[i]+b*(t-tvec[i])
    
    for(j in ind:i){
      newmax = eyz[j]+b*(t-tvec[j])
      if(newmax > max) max = newmax
    }
  
    sup[i-ind+1] = max
  }

  
  if(ind==1){
  u_comp1 = 0	
  u_comp2 = trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
  l_comp1 = 0
  l_comp2 = trapz(tvec[ind:length(tvec)],sup*fz[ind:length(tvec)])
  }else if(ind==length(tvec)){
  u_comp1 = trapz(tvec[1:ind],inf*fz[1:ind])	
  u_comp2 = 0
  l_comp1 = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind])
  l_comp2 = 0
  }else{
  u_comp1 = trapz(tvec[1:ind],inf*fz[1:ind])	
  u_comp2 = trapz(tvec[ind:length(tvec)],eyz[ind:length(tvec)]*fz[ind:length(tvec)])
  l_comp1 = trapz(tvec[1:ind],eyz[1:ind]*fz[1:ind])
  l_comp2 = trapz(tvec[ind:length(tvec)],sup*fz[ind:length(tvec)])
  
  }
  
  ub = u_comp1 + u_comp2  
  lb = l_comp1 + l_comp2
  res = c(lb,ub)
  res	
}



SMTRMTS1 = function(b,tvec,t,y,z,eyz,fz,wgt){ #SMTR-MTS bounds using sample analogue as much as possible

  
  ind = which(tvec==t)
  inf = numeric(ind)
  
  for(i in 1:ind){
    min = eyz[i]+b*(t-tvec[i])
    
    for(j in i:ind){
      newmin = eyz[j]+b*(t-tvec[j])
      if(newmin<min) min=newmin
    }
    
    inf[i] = min
  }  

  
  sup = numeric(length(tvec[ind:length(tvec)]))
  
  for(i in ind:length(tvec)){
    max = eyz[i]+b*(t-tvec[i])
    
    for(j in ind:i){
      newmax = eyz[j]+b*(t-tvec[j])
      if(newmax > max) max = newmax
    }
  
    sup[i-ind+1] = max
  }

  
  if(ind==1){
  u_comp1 = 0	
  u_comp2 = weighted.mean(y[z>=t],w=wgt)*weighted.mean(z>=t,w=wgt)
  l_comp1 = 0
  l_comp2 = trapz(tvec[ind:length(tvec)],sup*fz[ind:length(tvec)])
  }else if(ind==length(tvec)){
  u_comp1 = trapz(tvec[1:ind],inf*fz[1:ind])	
  u_comp2 = 0
  l_comp1 = weighted.mean(y[z<=t],w=wgt)*weighted.mean(z<=t,w=wgt)
  l_comp2 = 0
  }else{
  u_comp1 = trapz(tvec[1:ind],inf*fz[1:ind])	
  u_comp2 = weighted.mean(y[z>t],w=wgt)*weighted.mean(z>t,w=wgt)
  l_comp1 = weighted.mean(y[z<t],w=wgt)*weighted.mean(z<t,w=wgt)
  l_comp2 = trapz(tvec[ind:length(tvec)],sup*fz[ind:length(tvec)])
  
  }
  
  ub = u_comp1 + u_comp2  
  lb = l_comp1 + l_comp2
  res = c(lb,ub)
  res	
}


SMTRMTS_TE = function(b,tvec,t1,t2,eyz,fz){

	ind1 = which(tvec==t1)
	ind2 = which(tvec==t2)

	fi = function(s){

		inds = which(tvec==s)
		res = eyz[inds]+b*(t2-s)

		for(i in inds:ind2){
			newres = eyz[i]+b*(t2-tvec[i])
			res = min(res,newres)
		}

		res

	}

	fs = function(s){

		inds = which(tvec==s)
		res = eyz[ind1]

		for(i in ind1:inds){
			newres = eyz[i]-b*(tvec[i]-t1)
			res = max(res,newres)
		}

		res
	}

	comp1 = numeric(length(tvec[1:ind1]))

	for(i in 1:ind1){

		comp1[i] = min(fi(tvec[i])-eyz[i],b*(t2-t1))

	}

	comp2 = numeric(length(tvec[ind1:ind2]))

	for(i in ind1:ind2){

		comp2[i-ind1+1] = min(fi(tvec[i])-fs(tvec[i]),b*(t2-t1))

	}

	comp3 = numeric(length(tvec[ind2:length(tvec)]))

	for(i in ind2:length(tvec)){

		comp3[i-ind2+1] = min(eyz[i]-fs(tvec[i]),b*(t2-t1))

	}

	ub = trapz(tvec[1:ind1],comp1*fz[1:ind1])+trapz(tvec[ind1:ind2],comp2*fz[ind1:ind2])+trapz(tvec[ind2:length(tvec)],comp3*fz[ind2:length(tvec)])

	res = ub

	res

}