### plotAll calculates all the bounds and plots the results

plotAll <- function(data, flag, b, ey0, emp.var, misc = NULL){ 
  
  cat("Working on (", flag, "/",ifelse(emp.var,"emp )...\n","wage)...\n"))
  cat("EY0 =",ey0,"\n")
  cat("n =",nrow(data),"\n")
  
  fileid <- ifelse(emp.var, paste0("-emp-",flag,misc,".pdf"), paste0("-wage-",flag,misc,".pdf")) # filename identifier
  
  Gamma <- 40 #right boundary of Gamma (the region where our assumptions hold)
  t <- round(seq(0.1,Gamma,by = 0.1),1) #evaluation grid
  t.n <- length(t)
  
  eyz <- loess(y~z, data=data, weights=wgt48b) # LOESS with degree = 2 and span = 0.75
  fz <- density(data$z, bw = "nrd", from = 0.1, to = Gamma, n = t.n, kernel = "gaussian", weights = data$wgt48b/sum(data$wgt48b))
  # KDE with Silverman BW


  eyz <- predict(eyz, data.frame(z=t))
  
  ## plot sorted and unsorted E[Y|Z]
  pdf(paste0("EYZ",fileid))
  par(mar=c(5.1,5.1,2.1,2.1))
  grid <-range(eyz)
  ylabel <- ifelse(emp.var,"Employment Rate", "Average Weekly Wage")
  plot(t,eyz, type="l",xlab="Weeks of Training",ylab=ylabel, yaxt="n", cex.axis = 1.8, cex.lab = 1.8)
  if(emp.var){axis(2, at=pretty(eyz), lab=paste0(pretty(eyz * 100),"%"), cex.axis = 1.8)}
  dev.off()  
  
  eyz <- sort(eyz) #rearrange the conditional means
  
  ## plot kernel density estimator
  pdf(paste0("fz",fileid)) # plot kernel density estimator
  par(mar=c(5.1,5.1,2.1,2.1))
  grid <-range(fz$y)
  plot(t,fz$y, ylim = c(0,0.03), type="l",xlab="Weeks of Training",ylab="Density", cex.axis = 1.8, cex.lab = 1.8)
  dev.off()  
  
  cat("Bandwidth for Kernel Density Estimator:", fz$bw,"\n" )
  
  cat("P(z<=40) =",trapz(t[1:400],fz$y[1:400]),"\n")
  fz$y <- (fz$y)/trapz(t[1:400],fz$y[1:400]) 

  ##########################
  ## Obtaining the bounds ##
  ##########################	
  
  ymax <- max(data$y)
  ymin <- min(data$y)
 
  ## plot range
  plott <- 1:Gamma
  
  ## store the results in the following matrices
  mtrres <- matrix(0,Gamma,2)
  smtrres <- matrix(0,Gamma,2)
  mtrsres <- matrix(0,Gamma,2)
  smtrsres <- matrix(0,Gamma,2)

  ## calculate the bounds
  for(i in plott){
    mtrres[i,] <- MTR(t,i,eyz,fz$y,ymax,ymin)
    smtrres[i,] <- SMTR(b,t,i,eyz,fz$y)
    mtrsres[i,] <- MTRMTS(t,i,eyz,fz$y)
    smtrsres[i,] <- SMTRMTS(b,t,i,eyz,fz$y)
  }
  

  y.axis <- ifelse(rep(emp.var,2),c(-0.05,0.1),c(-10,40))
  
  ## plot MTR-MTS vs. SMTR-MTS
  pdf(paste0("mtr-smtr-sel",fileid)) 
  
  yaxt.mark <- ifelse(emp.var,"n","s")
  
  par(mar=c(5.1,5.1,2.1,2.1))
  plot(plott,mtrsres[ ,2]-ey0,lty=1, type="l",xlab="Weeks of Training", ylab= ylabel, ylim = y.axis, cex.axis = 1.8, cex.lab = 1.8, yaxt=yaxt.mark)
  if(emp.var){axis(2, at=pretty(y.axis), lab=paste0(pretty(y.axis * 100),"%"), cex.axis = 1.8)}
  lines(plott,mtrsres[ ,1]-ey0,lty=1) 
  lines(plott,smtrsres[ ,2]-ey0,lty=2) 
  lines(plott,smtrsres[ ,1]-ey0,lty=2) 
  abline(h=0, col = "red", lty = 2)
  legend("topleft",c("MTR-MTS","SMTR-MTS"),lty=c(1,2))
  
  dev.off()
  

  if(emp.var){
    mtrres <- matrix(round(100*(mtrres-ey0),2),40,2)
    smtrres <- matrix(round(100*(smtrres-ey0),2),40,2)
    mtrsres <- matrix(round(100*(mtrsres-ey0),2),40,2)
    smtrsres <- matrix(round(100*(smtrsres-ey0),2),40,2)
  }
  
  if(!emp.var){
    mtrres <- mtrres - ey0
    smtrres <- smtrres - ey0
    mtrsres <- mtrsres - ey0
    smtrsres <- smtrsres - ey0
  }
  
  tab <- data.frame((mtrres)[c(4,16,36),1],(smtrres)[c(4,16,36),1],
                    (mtrsres)[c(4,16,36),1],(smtrsres)[c(4,16,36),1],
                    (mtrres)[c(4,16,36),2],(smtrres)[c(4,16,36),2],
                    (mtrsres)[c(4,16,36),2],(smtrsres)[c(4,16,36),2])
  rownames(tab) <- c("4", "16", "36")
  write(print(xtable(tab,
                     caption = paste0("(",flag,", ",ifelse(emp.var,"emp","wage"),")")),
                     print.results=FALSE)
              ,"tables.tex",append = "TRUE") 
} 