rm(list=ls())

library(caTools)
library(locpol)
library(xtable)


source("./Functions/bound_functions.R")

# Importing and clearing data

master.data <- read.csv("../Data/CleanedJCData.csv",header = TRUE, as.is = T) #master data
master.data$earnq16 <- as.numeric(master.data$earnq16)
master.data <- master.data[!is.na(master.data$earnq16),]
master.data <- master.data[!is.na(master.data$z),]

master.data$y <- master.data$earnq16

n <- nrow(master.data)

attach(master.data)

# Range of smoothness parameters to use
bgrid=seq(0.5,3.0,by=0.1)

# Data storage
resultmat = array(0,dim=c(length(bgrid)+1,6,3))
resultmat_TE = array(0,dim=c(length(bgrid)+1,2,3))

# Which t's to conduct treatment effect analysis
t1 <- 4 #Q10 = 3
t2 <- 16 #Q50 = 15
t3 <- 36 #Q90 = 35.25

tpoint = c(t1,t2,t3)

# Calculating and storing TE bounds
for(c in 1:3){

	choice = c # 1 when analyzing full sample, 2 when analyzing male only, 3 when analyzing female only

	if(choice==1){
	tcomp=treatmnt==1&z>0
	ccomp=treatmnt==0&z==0
	}else if(choice==2){
	tcomp=treatmnt==1&z>0&female==0
	ccomp=treatmnt==0&z==0&female==0
	}else if(choice==3){
	tcomp=treatmnt==1&z>0&female==1
	ccomp=treatmnt==0&z==0&female==1
	}

	ty = y[tcomp]
	tz = z[tcomp]
	cy = y[ccomp]

	twgt48b = wgt48b[tcomp]

	ey0 = weighted.mean(cy,w=wgt48b[ccomp])


	tvec = seq(from=0.1,to=40,by=0.1)

	#Nonparametric regression for E[Y|Z]
	eyz <- loess(ty~tz, weights=twgt48b)
	eyz <- predict(eyz, data.frame(tz=tvec))
	eyz <- sort(eyz) #We will use sorted values of E[Y|Z] - see Chernozhukov, Victor, Ivan Fernandez-Val, and Alfred Galichon. "Improving point and interval estimators of monotone functions by rearrangement." Biometrika (2009)

	fz <- density(tz, bw = "nrd", from = 0.1, to = 40, n = length(tvec), kernel = "gaussian", weights = twgt48b/sum(twgt48b))
	fz <- fz$y/trapz(tvec,fz$y[1:length(tvec)]) 

	tvec = seq(from=0.1,to=40,by=0.1)

	res=array(0,dim=c(length(bgrid)+1,2,3))


	#MTRMTS = function(tvec,t,eyz,fz)
	#MTRMTS1 = function(tvec,t,y,z,eyz)
	#SMTRMTS = function(b,tvec,t,eyz,fz)
	#SMTRMTS1 = function(b,tvec,t,y,z,eyz,fz)

	for(j in 1:2){

		for(k in 1:3){
		
			for(b in 1:length(bgrid)){
				
				res[(b+1),j,k]=SMTRMTS(bgrid[b],tvec,tpoint[k],eyz,fz)[j]
				
			}
			res[1,j,k]=MTRMTS(tvec,tpoint[k],eyz,fz)[j]

		}
	}
	
	for(b in 1:length(bgrid)){
	  
	  resultmat_TE[(b+1),1,c]=SMTRMTS_TE(bgrid[b],tvec,t1,t2,eyz,fz)
	  resultmat_TE[(b+1),2,c]=SMTRMTS_TE(bgrid[b],tvec,t2,t3,eyz,fz)
	  
	}
	
	resultmat_TE[1,1,c]=res[1,2,2]-res[1,1,1]
	resultmat_TE[1,2,c]=res[1,2,3]-res[1,1,2]

	res=res-ey0

	mat1 = res[,,1]
	mat2 = res[,,2]
	mat3 = res[,,3]

	resultmat[,,c] = cbind(mat1,mat2,mat3)
	
}


############Numbers for Table 2, A-4, A-5###############

# This loop generates numbers for average weekly wage in Table 2 and A-4, A-5 in the order of 
# 1) all individuals sample (Table 2 and A-5)
# 2) male sample (Table A-4 and A-5)
# 3) female sample (Table A-4 and A-5)

clist = c("all","male","fem")

tol = 0.0001

TEresmat=numeric(0)

for(c in 1:3){
  
  mat = resultmat[,,c]
  mat_TE = resultmat_TE[,,c]
  
  rownames(mat)=c("MTRMTS",as.character(bgrid))
  rownames(mat_TE)=c("MTRMTS",as.character(bgrid))
  colnames(mat)=c("LB","UB","LB","UB","LB","UB")
  colnames(mat_TE)=c("16-4","36-16")
  truncind=c(1,2,7,12,17,22,27)  # Only for b=0.001, 0.0015, 0.002,0.0025,0.003,...
  print(c("Table for ",clist[c]))
  table=xtable(mat[truncind,],digits=2)
  print(table)
  print(c("TE table for ",clist[c]))
  table_TE=xtable(mat_TE[truncind,2,drop=FALSE],digits=2)
  print(table_TE)
  TEresmat = cbind(TEresmat,mat_TE[truncind,2,drop=FALSE])
  
  #effective region calculation for t3-t2 (36-16)
  
  trivub = bgrid*(t3-t2)
  nontrivind = which((mat_TE[2:(length(bgrid)+1),2])-trivub<(-tol))
  from = ifelse(length(nontrivind)!=0,bgrid[min(nontrivind)],99999)
  nontrivind = which((mat_TE[2:(length(bgrid)+1),2]-mat_TE[1,2])<(-tol))
  to = ifelse(length(nontrivind)!=0,bgrid[max(nontrivind)],-99999)
  print(paste("Effective region for",clist[c],"is from",as.character(from),"to",as.character(to)))
  
}





###############Figures for Figure 5, A-4, A-6################


fontsize = 1.8

bba = 1.13
bbm = 1.3
bbf = 0.92
bblist=c(bba,bbm,bbf)

yran = c(-10,50)

for(c in 1:3){
  
  for(t in 1:3){ # Plots in Figure A-6
    
    filename=paste(clist[c],as.character(tpoint[t]),".pdf",sep="")
    pdf(filename)
    plot(bgrid,resultmat[,,c][2:(length(bgrid)+1),2*t-1],type="l",ylim=yran,cex.lab=fontsize,cex.axis=fontsize,xlab="Smoothness parameters",ylab="")
    lines(bgrid,resultmat[,,c][2:(length(bgrid)+1),2*t])
    abline(h=resultmat[,,c][1,2*t-1], col = "red", lty = 2)
    abline(h=resultmat[,,c][1,2*t], col = "red", lty = 2)
    abline(v=bblist[c],col="blue",lty=2)
    title(ylab="Average Weekly Wage", cex.lab = fontsize,line = 2.5)
    dev.off()
    
  }
  
  
  for(i in 1:2){ # Plots in Figure 5 and A-4
    
    yran_TE=c(0,40)
    filename=paste(clist[c],as.character(tpoint[i+1]),"-",as.character(tpoint[i]),".pdf",sep="")
    pdf(filename)
    plot(bgrid,resultmat_TE[,i,c][2:(length(bgrid)+1)],type="l",ylim=yran_TE,cex.lab=fontsize,cex.axis=fontsize,xlab="Smoothness parameters",ylab="")
    abline(h=resultmat_TE[1,i,c], col = "red", lty = 2)
    lines(bgrid,(tpoint[i+1]-tpoint[i])*bgrid,col="red",lty=4)
    title(ylab="Average Weekly Wage", cex.lab = fontsize,line = 2.5)
    abline(v=bblist[c],col="blue",lty=2)
    dev.off()
    
  }
}
